@@ -12,8 +12,11 @@
  *   @include "/sieve/src/sieve@mozdev.org/chrome/chromeFiles/content/libs/libManageSieve/SieveRequest.js"   
  */
  
-const Cc = Components.classes;
-const Ci = Components.interfaces;
+if (typeof(Cc) == 'undefined')
+  { Cc = Components.classes; }
+
+if (typeof(Ci) == 'undefined')
+  { Ci = Components.interfaces; } 
 
 var gSid = null;
 var gCid = null;
@@ -25,7 +28,27 @@ var gPrintSettings = null;
 
 var gEditorStatus =
 {
+  insertString: function (text, select)
+  {
+    var txtScript = document.getElementById("sivContentEditor");
+    
+    var params = Cc["@mozilla.org/embedcomp/command-params;1"].createInstance(Ci.nsICommandParams);
+    params.setStringValue("state_data",text);  
   
+    txtScript.controllers
+        .getControllerForCommand("cmd_insertText")
+        .QueryInterface(Ci.nsICommandController)
+        .doCommandWithParams("cmd_insertText",params);
+    
+    txtScript.focus();
+    
+    if (!select)
+      return;
+      
+    var selStart = txtScript.selectionStart;  
+    txtScript.setSelectionRange(selStart - text.length, selStart);
+  },
+
   selectionStart    : -1,
   selectionEnd      : -1,
   selectionChanged  : false,
@@ -44,6 +67,43 @@ var gEditorStatus =
 
 var event = 
 {
+  
+  onChannelCreated : function(sieve)
+  { 
+    event.onChannelReady(gCid);
+  },
+  
+  onChannelReady : function(cid)
+  {
+    // We observe only our channel...
+    if (gCid != cid)
+      return;
+      
+    if (gEditorStatus.defaultScript)
+    {
+      event.onScriptLoaded(gEditorStatus.defaultScript);
+      gEditorStatus.contentChanged = true;
+      return;
+    }
+      
+    var request = new SieveGetScriptRequest(document.getElementById("txtName").value);
+    request.addGetScriptListener(event);
+    request.addErrorListener(event);
+
+    sivSendRequest(gSid,gCid,request);    
+  },
+  
+  onChannelClosed : function(cid)
+  {
+    // some other channel died we don't care about that...
+  },
+  
+  onChannelStatus : function(id,text,statusbar)
+  {
+    //TODO Move First run wizard, badcert warning etc to a separate
+    // xul file so that both files can share the same connection logic.
+  },
+  
   /**
    * @param {SieveGetScriptResponse} response
    */
@@ -57,12 +117,18 @@ var event =
    */
   onScriptLoaded: function(script)
   {
+    
     gEditorStatus.hasContent = true;
     sivSetStatus(0);
+    document.getElementById("sivContentEditor").editor.enableUndo(false);
     document.getElementById("sivContentEditor").value = script;
     document.getElementById("sivContentEditor").setSelectionRange(0, 0);
-	  UpdateCursorPos();
+    document.getElementById("sivContentEditor").editor.enableUndo(true);
+  
+    UpdateCursorPos();
     UpdateLines();
+    
+    document.getElementById("sivContentEditor").focus();    
   },
 
   /**
@@ -72,12 +138,12 @@ var event =
   {
     gEditorStatus.contentChanged = false;
 
-    if (gEditorStatus.isClosing && onClose())
+    if (gEditorStatus.isClosing && onWindowClose())
       close();
   },
 
   /**
-   * @param {SieveAbstractResponse} response
+   * @param {SieveSimpleResponse} response
    */
   onError: function(response)
   {
@@ -89,13 +155,18 @@ var event =
     sivSetStatus(2);
   },
   
+  onDisconnect:function(response)
+  {
+    sivSetStatus(3);
+  },
+  
   observe : function(aSubject, aTopic, aData)
   {
     if (aTopic != "quit-application-requested")
       return;
     
       
-    if (onClose() == false)
+    if (onWindowClose() == false)
       aSubject.QueryInterface(Ci.nsISupportsPRBool).data = true;
     else
       close();
@@ -124,7 +195,7 @@ function onCompile()
     {
       // TODO: The response might contain warnings, parse them
       document.getElementById("lblErrorBar").firstChild.nodeValue
-        = "Server reports no script errors...";
+        = document.getElementById("strings").getString("syntax.ok");
 
       document.getElementById("imgErrorBar").src
         = "chrome://sieve/content/images/syntax-ok.png";      
@@ -142,8 +213,9 @@ function onCompile()
       // we got an overquota warning, this means syntaxcheck can't be performed
       if (response.getResponseCode().equalsCode("QUOTA"))
       {
+        
         document.getElementById("lblErrorBar").firstChild.nodeValue
-          = "Server has not enough free resources to perform a syntax check"
+          = document.getElementById("strings").getString("syntax.quota");
           
         document.getElementById("imgErrorBar").src
           = "chrome://sieve/content/images/syntax-warning.png";
@@ -178,14 +250,13 @@ function onCompile()
   
   var canCheck = Cc["@sieve.mozdev.org/transport-service;1"]
                    .getService().wrappedJSObject  
-                   .getChannel(gSid,gCid)
-                   .getCompatibility()
-                   .checkscript;                
+                   .getChannel(gSid,gCid).getCompatibility()
+                   .checkscript;
 
   if (canCheck)
   {
     // ... we use can the CHECKSCRIPT command
-    request = new SieveCheckScriptRequest(script)
+    request = new SieveCheckScriptRequest(script);
     request.addCheckScriptListener(lEvent);
   }
   else
@@ -194,7 +265,7 @@ function onCompile()
     
     // First we use PUTSCRIPT to store a temporary script on the server...
     // ... incase the command fails, it is most likely due to an syntax error...
-    // ... if it sucseeds the script is syntactically correct! 
+    // ... if it sucseeds the script is syntactically correct!
     request = new SievePutScriptRequest("TMP_FILE_DELETE_ME",script);
     request.addPutScriptListener(lEvent);
   }
@@ -226,8 +297,44 @@ function onInput()
   UpdateLinesLazy();
 }
 
-function onLoad()
+function onEditorKeyDown(event)
+{
+  // we need this bypass the default onKeyDown only for the tab key...
+  if (event.keyCode != 9)
+    return;
+ 
+  if (event.shiftKey || event.ctrlKey || event.altKey || event.metaKey)
+    return;
+    
+  gEditorStatus.insertString(String.fromCharCode(9));
+           
+  event.preventDefault();
+  
+  onInput();
+  //event.stopPropagation();
+}
+
+function onWindowPersist()
+{
+  // we just mirror the open dialogs
+  var args = {};
+  
+  if (gEditorStatus.contentChanged)
+  {
+    args["scriptBody"] = document.getElementById("sivContentEditor").value;
+    args["contentChanged"] = gEditorStatus.contentChanged;
+  }
+    
+  args["scriptName"] = document.getElementById("txtName").value;
+  args["compile"] = document.getElementById('btnCompile').checked;
+  args["account"] = gEditorStatus.account;  
+  
+  return args; 
+}
+
+function onWindowLoad()
 { 
+        
   // checkbox buttons are buggy in Gecko 1.8, this has been fixed in ...
   // ...Gecko 1.9 (Thunderbird 3).
   // We implement the workaround mentioned in Bug 382457.
@@ -250,23 +357,9 @@ function onLoad()
       function() {onSearchBar();},
       false);
  
-  // Gecko 1.8.1 does not propagate onscroll events for textboxes...
-  // ... the bug is fixed in Gecko 1.9 (Thunderbird 3)
-  // As the is no workaround, we simply deactivate line numbers in Gecko 1.8
-  var version  = Components.classes["@mozilla.org/xre/app-info;1"]
-                              .getService(Components.interfaces.nsIXULAppInfo)
-                              .platformVersion.split(".");
-
-  if (((version[0] == 1) && (version[1] >=9)) || version[0] >=2)
-  {
-    document.getElementById("sivLineNumbers").removeAttribute('hidden');
-    document.getElementById("sivContentEditor")
-        .addEventListener("scroll", function() {onEditorScroll();},false);
-  }
-  else
-  {
-    document.getElementById("sivLineNumbers").setAttribute('hidden', 'true');
-  }
+  document.getElementById("sivLineNumbers").removeAttribute('hidden');
+  document.getElementById("sivContentEditor")
+      .addEventListener("scroll", function() {onEditorScroll();},false);
 
   // add event listeners to Editor
   document.getElementById("sivContentEditor")
@@ -277,66 +370,59 @@ function onLoad()
       
   document.getElementById("sivContentEditor")
       .addEventListener("keypress",function() {onUpdateCursorPos(50);},false);
-      
+
+  document.getElementById("sivContentEditor")
+       .addEventListener("keydown", function(ev) { onEditorKeyDown(ev)},true);
   // hack to prevent links to be opened in the default browser window...
   document.getElementById("ifSideBar").
     addEventListener(
       "click",
       function(event) {onSideBarBrowserClick(event);},
       false);
+
+  var args = window.arguments[0].wrappedJSObject;
   
-  var args = window.arguments[0].wrappedJSObject;    
-      
-  // Connect to the Sieve Object...  
-  var sivManager = Components.classes["@sieve.mozdev.org/transport-service;1"]
-                     .getService().wrappedJSObject; 
+  // There might be a default or persisted script...
+  if (args["scriptBody"])
+    gEditorStatus.defaultScript = args["scriptBody"];
   
-  gSid = args["sieve"];
-  gCid = sivManager.createChannel(gSid);
   
-  gEditorStatus.checkScriptDelay = args["compileDelay"];
+  gEditorStatus.account = args["account"];
+  var account = (new SieveAccounts()).getAccount(gEditorStatus.account);
+  
+  gEditorStatus.checkScriptDelay = account.getSettings().getCompileDelay();
   
   document.getElementById("txtName").value = args["scriptName"];
   document.title = ""+args["scriptName"]+" - Sieve Filters";
-  
-  document.getElementById("lblErrorBar").firstChild.nodeValue
-    = "Server reports no script errors...";
-  
-  if (args["scriptBody"] != null)
-  {
-    event.onScriptLoaded(args["scriptBody"]);
-  }
-  else
-  {
-    sivSetStatus(1,"Loading Script...");
-      
-    var request = new SieveGetScriptRequest(args["scriptName"]);
-    request.addGetScriptListener(event);
-    request.addErrorListener(event);
 
-    sivSendRequest(gSid,gCid,request);
-  }
+  document.getElementById("lblErrorBar").firstChild.nodeValue
+      = document.getElementById("strings").getString("syntax.ok");
+    
+  sivSetStatus(1,"status.loading");
+
+  // Connect to the Sieve Object...  
+  var sivManager = Cc["@sieve.mozdev.org/transport-service;1"]
+                     .getService().wrappedJSObject; 
   
+  gSid = sivManager.createSession(account.getKey());
+  sivManager.addSessionListener(gSid,event);
+  
+  gCid = sivManager.createChannel(gSid);
+  sivManager.openChannel(gSid,gCid);
+    
   //preload sidebar...
   onSideBarHome();
   
+  if (!args["compile"])
+    args["compile"] = account.getSettings().hasCompileDelay();
+
   onErrorBar(args["compile"]);
   onSideBar(true);
   onSearchBar(false);
-  
-  document.getElementById("sivContentEditor").setSelectionRange(0, 0);
-  document.getElementById("sivContentEditor").focus();
 
   Cc["@mozilla.org/observer-service;1"]
       .getService (Ci.nsIObserverService)
-      .addObserver(event,"quit-application-requested", false);
-           
-  /*
-   * window.document.documentElement.addEventListener('focus', function(event) {
-   * if (event.target.nodeName=='textbox' &&
-   * event.target.hasAttribute('readonly')) {
-   * window.document.documentElement.focus(); } }, true);
-   */
+      .addObserver(event,"quit-application-requested", false);           
 }
 
 function onIgnoreOffline()
@@ -344,15 +430,21 @@ function onIgnoreOffline()
   // try to go online again
   try 
   {
+    var oldSid  = gSid;   
+               
     // Cid is removed, but try to reconnect...
     gCid = Cc["@sieve.mozdev.org/transport-service;1"]
                .getService().wrappedJSObject
                .createChannel(gSid);
+               
+    Cc["@sieve.mozdev.org/transport-service;1"]
+               .getService().wrappedJSObject
+               .closeChannel(oldSid);               
     
     // TODO: this is code exists twice remove me...
     if (gEditorStatus.hasContent == false)
     {
-      sivSetStatus(1,"Loading Script...");
+      sivSetStatus(1,"status.loading");
       
       var args = window.arguments[0].wrappedJSObject;
       
@@ -364,13 +456,32 @@ function onIgnoreOffline()
       
       return;
     }
-      
   } 
   catch (ex) {}
   
   sivSetStatus(0);
 }
 
+
+function onReconnectClick()
+{
+  gEditorStatus.defaultScript = document.getElementById("sivContentEditor").value;
+  
+  var account = (new SieveAccounts()).getAccount(gEditorStatus.account);
+  
+  sivSetStatus(1,"status.loading");
+
+  // Connect to the Sieve Object...  
+  var sivManager = Cc["@sieve.mozdev.org/transport-service;1"]
+                     .getService().wrappedJSObject; 
+  
+  gSid = sivManager.createSession(account.getKey());
+  sivManager.addSessionListener(gSid,event);
+  
+  gCid = sivManager.createChannel(gSid);
+  sivManager.openChannel(gSid,gCid);   
+}
+
 function onSideBarBrowserClick(event)
 {     
   var href = null;
@@ -460,7 +571,7 @@ function onSave()
   sivSendRequest(gSid,gCid,request);
 }
 
-function onClose()
+function onWindowClose()
 {
   if (gEditorStatus.contentChanged == true)
   {
@@ -488,9 +599,13 @@ function onClose()
    
   clearTimeout(gEditorStatus.checkScriptTimer);
   
-  Cc["@mozilla.org/observer-service;1"]
-      .getService (Ci.nsIObserverService)
-      .removeObserver(event,"quit-application-requested");  
+  try
+  {
+    Cc["@mozilla.org/observer-service;1"]
+        .getService (Ci.nsIObserverService)
+        .removeObserver(event,"quit-application-requested");
+  }
+  catch (ex) {}
   
   // either the script has not changed or the user did not want to save... 
   // ... the script, so it's ok to exit.
@@ -511,10 +626,10 @@ function onImport()
   if (filePicker.show() != filePicker.returnOK)
     return;
 
-  var inputStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
-                              .createInstance(Components.interfaces.nsIFileInputStream);
-  var scriptableStream = Components.classes["@mozilla.org/scriptableinputstream;1"]
-                                   .createInstance(Components.interfaces.nsIScriptableInputStream);
+  var inputStream = Cc["@mozilla.org/network/file-input-stream;1"]
+                        .createInstance(Ci.nsIFileInputStream);
+  var scriptableStream = Cc["@mozilla.org/scriptableinputstream;1"]
+                             .createInstance(Ci.nsIScriptableInputStream);
 
   inputStream.init(filePicker.file, 0x01, 0444, null);
   scriptableStream.init(inputStream);
@@ -540,8 +655,8 @@ function onImport()
 
 function onExport()
 {
-  var filePicker = Components.classes["@mozilla.org/filepicker;1"]
-			.createInstance(Components.interfaces.nsIFilePicker);
+  var filePicker = Cc["@mozilla.org/filepicker;1"]
+			.createInstance(Ci.nsIFilePicker);
 
 	filePicker.defaultExtension = ".siv";
 	filePicker.defaultString = document.getElementById("txtName").value
@@ -560,10 +675,10 @@ function onExport()
 	var file = filePicker.file;
 
 	if (file.exists() == false)
-		file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0644);
+		file.create(Ci.nsIFile.NORMAL_FILE_TYPE, 0644);
 
-	var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
-			.createInstance(Components.interfaces.nsIFileOutputStream);
+	var outputStream = Cc["@mozilla.org/network/file-output-stream;1"]
+			.createInstance(Ci.nsIFileOutputStream);
 
 	outputStream.init(file, 0x04 | 0x08 | 0x20, 0644, null);
 
@@ -732,14 +847,15 @@ function OnFindString()
   {
     result = script.indexOf(token, position);
   }
-
+  
   // start search from cursor pos...
   if (result == -1)
   {
-    alert('Phrase not found...')
+    document.getElementById("boxSearchError").removeAttribute('hidden');
     return -1;
   }
 
+  document.getElementById("boxSearchError").setAttribute('hidden','true');
   txtScript.focus();
 
   txtScript.setSelectionRange(result, result + token.length);
@@ -765,17 +881,8 @@ function OnReplaceString()
   if (selectedToken != token)
     return;
   
-  var newToken = new String(document.getElementById("txtReplace").value);
-  var selStart = txtScript.selectionStart;
-  var selEnd = txtScript.selectionEnd
-  /* Remember obj is a textarea or input field */
-  txtScript.value = txtScript.value.substr(0, selStart) + newToken
-    + txtScript.value.substr(selEnd, txtScript.value.length);
-  
-  txtScript.focus();
-  
-  txtScript.setSelectionRange(selStart, selStart + newToken.length);
-  txtScript.editor.selectionController.scrollSelectionIntoView(1, 1, true);
+  var replace = document.getElementById("txtReplace").value;
+  gEditorStatus.insertString(replace,true);
   
   this.onInput();
   
@@ -1136,15 +1243,18 @@ function sivSetStatus(state, message)
 {
   document.getElementById('sivEditorWarning').setAttribute('hidden','true');
   document.getElementById('sivEditorWait').setAttribute('hidden','true');
+  document.getElementById('sivExplorerConnectionLost').setAttribute('hidden','true');
   document.getElementById('sivEditor').setAttribute('collapsed','true');
-  
+    
   switch (state)
   {
+    case 3: document.getElementById('sivExplorerConnectionLost').removeAttribute('hidden');
+            break;    
     case 2: document.getElementById('sivEditorWarning').removeAttribute('hidden');
             break;    
     case 1: document.getElementById('sivEditorWait').removeAttribute('hidden');
-            document.getElementById('sivEditorWaitMsg')
-                .firstChild.nodeValue = message;    
+            document.getElementById('sivEditorWaitMsg').firstChild.nodeValue 
+                = document.getElementById("strings").getString(message);
             break;
     case 0: document.getElementById('sivEditor').removeAttribute('collapsed');
             break;
