@@ -16,11 +16,14 @@
  *   @include "/sieve/src/sieve@mozdev.org/chrome/chromeFiles/content/editor/SieveFilterTreeView.js"
  */
 
-const Cc = Components.classes;
-const Ci = Components.interfaces;
+if (typeof(Cc) == 'undefined')
+  { Cc = Components.classes; }
+
+if (typeof(Ci) == 'undefined')
+  { Ci = Components.interfaces; } 
 
 var sid = null;
-var cid = null;
+var gCid = null;
 
 /** @type {{Components.interfaces.nsIConsoleService}}*/
 var gLogger = null;
@@ -53,7 +56,7 @@ var event =
     request.addListScriptListener(event);
     request.addErrorListener(event);
      
-    sivSendRequest(sid,cid,request);
+    sivSendRequest(sid,gCid,request);
   },
 
   onDeleteScriptResponse:  function(response)
@@ -63,20 +66,19 @@ var event =
     request.addListScriptListener(event);
     request.addErrorListener(event);
     
-    sivSendRequest(sid,cid,request);
+    sivSendRequest(sid,gCid,request);
   },
 
+  onOffline: function()
+  {
+    sivDisconnect(6);
+  },
+  
   onTimeout: function()
   {
-    var ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);  
-    
-    if (ioService.offline)
-    {
-      sivDisconnect(6);
-      return;
-    }
-    
     gLogger.logStringMessage("SivFilterExplorer.js\nOnTimeout");
+    
+    
     sivDisconnect(1,"warning.timeout");
   },
 	
@@ -86,6 +88,11 @@ var event =
     sivDisconnect(4,response.getMessage());
   },
   
+  onDisconnect: function()
+  {
+    sivDisconnect(9);
+  },
+  
   onCycleCell: function(row,col,script,active)
   {
   	var request = null;
@@ -97,7 +104,7 @@ var event =
     request.addSetActiveListener(event);
     request.addErrorListener(event);
 
-    sivSendRequest(sid,cid,request);
+    sivSendRequest(sid,gCid,request);
   },
       
   onChannelClosed : function()
@@ -109,17 +116,26 @@ var event =
     request.addListScriptListener(event);
     request.addErrorListener(event);
 
-    sivSendRequest(sid,cid,request);
+    sivSendRequest(sid,gCid,request);
   },
   
   onChannelCreated : function(sieve)
-  {    
+  {
+    event.onChannelReady(gCid);
+  },
+  
+  onChannelReady : function(cid)
+  {
+    // We observe only our channel...
+    if (gCid != cid)
+      return;
+      
     // List all scripts as soon as we are connected
     var request = new SieveListScriptRequest();
     request.addListScriptListener(event);
     request.addErrorListener(event);
 
-    sieve.addRequest(request);
+    sivSendRequest(sid,gCid,request);
   },
   
   onChannelStatus : function(id,text,statusbar)
@@ -138,7 +154,7 @@ var event =
       return;
     
     if (aData == "offline")
-      sivDisconnect(6);
+      this.onOffline();
     
     if (aData == "online")
       sivConnect();    
@@ -160,17 +176,13 @@ function onWindowLoad()
   var menuImapAccounts = document.getElementById("menuImapAccounts");
 
   var accounts = (new SieveAccounts()).getAccounts();
+  
 
   for (var i = 0; i < accounts.length; i++)
   {   
-    if (accounts[i].isEnabled() == false)
-      menuImapAccounts.appendItem(
-        accounts[i].getDescription(),
-        accounts[i].getKey(),"- disabled").disabled = true;
-    else
-      menuImapAccounts.appendItem( 
-        accounts[i].getDescription(),
-        accounts[i].getKey(),"").disabled = false;
+    menuImapAccounts.appendItem( 
+      accounts[i].getDescription(),
+      accounts[i].getKey(),"").disabled = false;
 
     if (window.arguments.length == 0)
       continue;
@@ -202,9 +214,16 @@ function onWindowClose()
   Cc["@mozilla.org/observer-service;1"]
       .getService (Ci.nsIObserverService)
       .removeObserver(event,"network:offline-status-changed");  
-      
+ 
+  if (gAutoConfig)
+  {
+    gAutoConfig.cancel();
+    gAutoConfig = null;
+  }
+    
   return true;
-}   
+}
+
 /**
  * @return {SieveAccount}
  */
@@ -230,6 +249,11 @@ function onActivateClick()
   return;
 }
 
+function onReconnectClick()
+{
+  sivConnect();   
+}
+
 function onGoOnlineClick()
 {
   var ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);  
@@ -253,12 +277,12 @@ function sivConnect(account)
   if (!account)
     account = getSelectedAccount();
  
-  sid = sivManager.createSession(account);
+  sid = sivManager.createSession(account.getKey());
   sivManager.addSessionListener(sid,event);
   
-  cid = sivManager.createChannel(sid);
+  gCid = sivManager.createChannel(sid);
   
-  sivManager.openChannel(sid,cid);
+  sivManager.openChannel(sid,gCid);
 }
 
 function sivSendRequest(sid,cid,request)
@@ -300,19 +324,25 @@ function sivDisconnect(state,message)
   if (state)
     sivSetStatus(state,message,"status.disconnected");  
   
-  if ((!sid) || (!cid))
+  if ((!sid) || (!gCid))
     return;
     
   var sivManager = Cc["@sieve.mozdev.org/transport-service;1"]
                        .getService().wrappedJSObject;
-  sivManager.removeSessionListener(sid);
-  sivManager.closeChannel(sid,cid);    
+  sivManager.removeSessionListener(sid, event);
+  sivManager.closeChannel(sid,gCid);    
 }
 
 function onSelectAccount()
-{      
+{
+  if (gAutoConfig)
+  {
+    gAutoConfig.cancel();
+    gAutoConfig = null;
+  }
+    
   sivDisconnect();
-      
+  
   // update the TreeView...
   var tree = document.getElementById('treeImapRules');
       
@@ -327,15 +357,28 @@ function onSelectAccount()
     return sivSetStatus(2,"error.noaccount");
       
   // Disable and cancel if account is not enabled
-  if (account.isEnabled() == false)
-    return sivSetStatus(1,"warning.noaccount");
+  if ((!account.isEnabled()) || account.isFirstRun())
+  {
+    account.setFirstRun();
+    return sivSetStatus(8);
+  }
     
  // TODO wait for timeout or session close before calling connect again
+ // otherwise we might endup using a closing channel. An isClosing function
+ // and the follwoing code migh help to detect this...
+ 
+/*  if (isClosing)
+  setTimeout(function(){sivDisconnect(force=true); sivConnect(account)}, 1000);*/
+    
   sivConnect(account);
 }
 
 function onDeleteClick()
 {
+  var tree = document.getElementById('treeImapRules');  
+  if (tree.currentIndex < 0)
+    return;
+    
   var prompts = Cc["@mozilla.org/embedcomp/prompt-service;1"]
                   .getService(Ci.nsIPromptService);
   	
@@ -353,11 +396,6 @@ function onDeleteClick()
   if (button != 0)
     return;
   
-  var tree = document.getElementById('treeImapRules');
-  
-  if (tree.currentIndex == -1)
-    return;
-  
   var scriptName = new String(tree.view.getCellText(tree.currentIndex, tree.columns.getColumnAt(0)));	
   
   // delete the script...
@@ -365,7 +403,7 @@ function onDeleteClick()
   request.addDeleteScriptListener(event);
   request.addErrorListener(event);
   
-  sivSendRequest(sid,cid,request);
+  sivSendRequest(sid,gCid,request);
 }
 /**
  * @param {String} scriptName
@@ -373,7 +411,7 @@ function onDeleteClick()
  */
 function sivOpenEditor(scriptName,scriptBody)
 {  
-  var wm = Cc["@mozilla.org/appshell/window-mediator;1"]  
+  /*var wm = Cc["@mozilla.org/appshell/window-mediator;1"]  
              .getService(Ci.nsIWindowMediator);
              
   var enumerator = wm.getEnumerator("Sieve:FilterEditor");  
@@ -389,24 +427,80 @@ function sivOpenEditor(scriptName,scriptBody)
       
     win.focus();    
     return;
-  }     
+  }     */
 
   var args = new Array();
   
   args["scriptName"] = scriptName;
-  args["scriptBody"] = scriptBody;
+  if (scriptBody)
+    args["scriptBody"] = scriptBody;
+    
   args["sieve"] = sid;
-  args["compile"] = getSelectedAccount().getSettings().hasCompileDelay();
-  args["compileDelay"] = getSelectedAccount().getSettings().getCompileDelay();
+  args["uri"] = "x-sieve:"+sid+"/"+scriptName;
+  args["account"] = this.getSelectedAccount().imapKey;
   
   // This is a hack from DEVMO
   args.wrappedJSObject = args;  
   
-  Cc["@mozilla.org/embedcomp/window-watcher;1"].getService(Ci.nsIWindowWatcher)
+  
+  /*Cc["@mozilla.org/embedcomp/window-watcher;1"].getService(Ci.nsIWindowWatcher)
       .openWindow(null,"chrome://sieve/content/editor/SieveFilterEditor.xul",
           "x-sieve:"+sid+"/"+scriptName, 
-          "chrome,titlebar,resizable,centerscreen,all", args);
+          "chrome,titlebar,resizable,centerscreen,all", args);*/
+
+  // Every script needs to be open in a unique tab...
+  // ... so we have to crawl through all windows through ... 
+  // ... looking for a tabmail component and test there if ...
+  // ... the script is already open.
+  var mediator = Components
+          .classes["@mozilla.org/appshell/window-mediator;1"]
+          .getService(Components.interfaces.nsIWindowMediator);
     
+  var w = mediator.getXULWindowEnumerator(null);
+    
+  while(w.hasMoreElements())
+  {
+    var win = w.getNext();
+    var docShells = win
+            .QueryInterface(Ci.nsIXULWindow).docShell
+            .getDocShellEnumerator(Ci.nsIDocShellTreeItem.typeChrome,Ci.nsIDocShell.ENUMERATE_FORWARDS);           
+        
+    while (docShells.hasMoreElements())
+    {
+      var childDoc = docShells.getNext()
+              .QueryInterface(Ci.nsIDocShell)
+              .contentViewer.DOMDocument;
+  
+      //if (childDoc.location.href == "chrome://sieve/content/editor/SieveFilterExplorer.xul")
+       
+      if (childDoc.location.href != "chrome://messenger/content/messenger.xul")
+        continue;
+         
+      var tabmail = childDoc.getElementById("tabmail");
+       
+      if (!tabmail)
+        continue;
+    
+      if (!tabmail.tabModes.SieveEditorTab)
+        continue;
+
+      if (!tabmail.tabModes.SieveEditorTab.tabs.length)
+        continue;
+      
+      for (var i = 0; i < tabmail.tabModes.SieveEditorTab.tabs.length ; i++)
+      {
+        if (tabmail.tabModes.SieveEditorTab.tabs[i].uri != args["uri"])
+          continue;
+
+        tabmail.switchToTab(tabmail.tabModes.SieveEditorTab.tabs[i]);
+        childDoc.defaultView.QueryInterface(Ci.nsIDOMWindow).focus();
+        
+        return;
+      } 
+    }
+  }
+  
+  window.tabmail.openTab("SieveEditorTab", args);
   return;  
 }
 
@@ -463,6 +557,9 @@ function sivSetStatus(state, message, statusbar)
   document.getElementById('sivExplorerWait').setAttribute('hidden','true');
   document.getElementById('sivExplorerBadCert').setAttribute('hidden','true');
   document.getElementById('sivExplorerOffline').setAttribute('hidden','true');
+  document.getElementById('sivExplorerDisabled').setAttribute('hidden','true');
+  document.getElementById('sivExplorerConnectionLost').setAttribute('hidden','true');
+  
   document.getElementById('sivExplorerTree').setAttribute('collapsed','true');
 
   if (statusbar)
@@ -493,10 +590,14 @@ function sivSetStatus(state, message, statusbar)
                 .firstChild.nodeValue = message;    
             break;            
     case 5: document.getElementById('sivExplorerBadCert').removeAttribute('hidden');
-            document.getElementById("btnIgnoreBadCert").setAttribute("oncommand",
-                "onBadCertOverride('"+message+"',document.getElementById('cbBadCertRemember').checked)");
-            document.getElementById("btnAbortBadCert").setAttribute("oncommand",
-                "sivSetStatus(1,'warning.brokencert')");            
+
+            document.getElementById("btnIgnoreBadCert").setAttribute("message", message);
+            document.getElementById("btnIgnoreBadCert").setAttribute("oncommand", 
+              "onBadCertOverride(this.getAttribute('message'),document.getElementById('cbBadCertRemember').checked);");
+              
+            document.getElementById("btnAbortBadCert").setAttribute("oncommand", 
+              "sivSetStatus(1,'warning.brokencert');");
+                            
             break;
     // Offline Mode
     case 6: document.getElementById('sivExplorerOffline').removeAttribute('hidden');
@@ -507,8 +608,13 @@ function sivSetStatus(state, message, statusbar)
             document.getElementById('txtVersion').value = "v"+message.getVersion().toFixed(2);
             document.getElementById('sivExplorerWait').removeAttribute('hidden');
             break;
+    // account disabled
+    case 8: document.getElementById('sivExplorerDisabled').removeAttribute('hidden');
+            document.getElementById('sivAutoConfig').selectedIndex = message;
+            break;  
+    case 9: document.getElementById('sivExplorerConnectionLost').removeAttribute('hidden');
+            break;
   }
-  
 }
 
 function disableControls(disabled)
@@ -546,7 +652,7 @@ function sivRename2(oldName, newName)
       request.addListScriptListener(event);
       request.addErrorListener(event);
   
-      sivSendRequest(sid,cid,request);           
+      sivSendRequest(sid,gCid,request);           
     },
     onTimeout: function()
     {
@@ -563,7 +669,7 @@ function sivRename2(oldName, newName)
   request.addRenameScriptListener(lEvent)
   request.addErrorListener(lEvent);
     
-  sivSendRequest(sid,cid,request);
+  sivSendRequest(sid,gCid,request);
 }
 
 function sivRename(oldName, newName, isActive)
@@ -583,7 +689,7 @@ function sivRename(oldName, newName, isActive)
       request.addPutScriptListener(lEvent)
       request.addErrorListener(lEvent)
       
-      sivSendRequest(sid,cid,request);
+      sivSendRequest(sid,gCid,request);
     },    
     onPutScriptResponse: function(response)
     {
@@ -595,7 +701,7 @@ function sivRename(oldName, newName, isActive)
         request.addSetActiveListener(lEvent);
         request.addErrorListener(event);
     
-        sivSendRequest(sid,cid,request);
+        sivSendRequest(sid,gCid,request);
       }
       else
         lEvent.onSetActiveResponse(null);
@@ -608,7 +714,7 @@ function sivRename(oldName, newName, isActive)
       request.addDeleteScriptListener(event);
       request.addErrorListener(event);
       
-      sivSendRequest(sid,cid,request);
+      sivSendRequest(sid,gCid,request);
     },
     onTimeout: function()
     {
@@ -640,7 +746,7 @@ function sivRename(oldName, newName, isActive)
   request.addGetScriptListener(lEvent);
   request.addErrorListener(event);
 
-  sivSendRequest(sid,cid,request);
+  sivSendRequest(sid,gCid,request);
   
 }
 
@@ -677,7 +783,7 @@ function onRenameClick()
 
   var canRename = Cc["@sieve.mozdev.org/transport-service;1"]
                     .getService().wrappedJSObject
-                    .getChannel(sid,cid)
+                    .getChannel(sid,gCid)
                     .getCompatibility().renamescript  
     
     
@@ -749,7 +855,68 @@ function onBadCertOverride(targetSite,permanent)
   catch (ex)
   {
     sivSetStatus(2,"error.brokencert");
-    gLogger.logStringMessage(ex); 
+    gLogger.logStringMessage("onBadCertOverride:"+ex); 
   }
  
 }
+
+/*
+ *  Auto Config
+ */ 
+
+var gAutoConfig = null;
+
+var gAutoConfigEvent =
+{
+  onSuccess : function(host,port,proxy)
+  {
+    sivSetStatus(8,2);
+    
+    getSelectedAccount().setActiveHost(0);
+    getSelectedAccount().getHost().setPort(port);
+    getSelectedAccount().setEnabled(true);
+    
+    gAutoConfig = null;
+  },
+
+  onError : function()
+  {
+    sivSetStatus(8,3);
+    gAutoConfig = null;
+  }
+}
+
+function onAutoConfigRunClick()
+{
+  if (gAutoConfig)
+    gAutoConfig.cancel();
+        
+  gAutoConfig = new SieveAutoConfig();
+  
+  gAutoConfig.addHost(
+    getSelectedAccount().getHost(0).getHostname(),
+    4190,
+    getSelectedAccount().getProxy().getProxyInfo());
+    
+  gAutoConfig.addHost(
+    getSelectedAccount().getHost(0).getHostname(),
+    2000,
+    getSelectedAccount().getProxy().getProxyInfo());
+  
+  gAutoConfig.run(gAutoConfigEvent);
+  
+  sivSetStatus(8,1);
+}
+
+function onAutoConfigCancelClick()
+{
+  gAutoConfig.cancel();
+  gAutoConfig = null;
+  
+  sivSetStatus(8,3);
+}
+
+function onAutoConfigFinishedClick()
+{
+  sivConnect();
+}
\ No newline at end of file
