@@ -5,150 +5,183 @@
  * 
  * The initial author of the code is:
  *   Thomas Schmid <schmid-thomas@gmx.net>
+ *   
+ * Contibutors:
+ *   Max Dittrich
  */
  
-/*******************************************************************************
- 
-  FACTSHEET: 
-  ==========
-    CLASS NAME          : SieveAbstractResponse
-    USES CLASSES        : SieveResponseParser
-        
-    CONSCTURCTOR        : SieveAbstractResponse(SieveResponseParser parser)
-    DECLARED FUNCTIONS  : String getMessage()
-                          String getResponse()
-                          int getResponseCode()                          
-                          boolean hasError()
-    EXCEPTIONS          : 
-    AUTHOR              : Thomas Schmid        
-    
-  DESCRIPTION:
-  ============
-    Every Sieve Response is tailed by eithern an OK, BYE or NO followed by 
-    optional messages and/or error codes.
-    This class is capable of parsing this part of the sieve response, therefore
-    it should be implemented by and SieveResponse.
+/**
+ * This class implements a generic response handler for simple sieve requests.
+ * 
+ * Simple requests just indicate, wether the command succeded or not. They 
+ * return only status information, and do not contain any data relevant for 
+ * the user.
+ *  
+ * @see SieveResponseParser
+ *  
+ * @param {SieveResponseParser} parser
+ *  a SieveResponseParser object containing the response sent by the server
+ */
 
-  PROTOCOL SNIPLET EXAMPLES: 
-  ==========================
-    'NO (0000) "Meldung"\r\n'
-    'BYE (0000) {4+}\r\n1234\r\n'
-    'NO \"Meldung\"\r\n'
-    'BYE {4+}\r\n1234\r\n'
-    'NO (0000)\r\n'
-    
-********************************************************************************/
-function SieveAbstractResponse(parser)
+function SieveSimpleResponse(parser)
 {
-    this.message = "";
-    this.responseCode = "";
-    
-    // OK
-    if (parser.startsWith([[79,111],[75,107]]))
-    {
-      this.response = 0;
-      parser.extract(2);
-    }
-    // BYE
-    else if (parser.startsWith([[66,98],[89,121],[69,101]]))
-    {
-      this.response = 1;
-      parser.extract(3);
-    }
-    // NO
-    else if (parser.startsWith([[78,110],[79,111]]))
-    {
-      this.response = 2;
-      parser.extract(2);
-    }
-    else
-      throw "NO, OK or BYE expected in"+parser.getData();
+  /*
+   * Examples for simple responses
+   * 
+   * 'NO (0000) "Message"\r\n'
+   * 'BYE (0000) {4+}\r\n1234\r\n'
+   * 'NO \"Message\"\r\n'
+   * 'BYE {4+}\r\n1234\r\n'
+   * 'NO (0000)\r\n'
+   */
 
-    // is there a Message?
-    if (parser.isLineBreak())
-      return;
-
-    // remove the space
-    parser.extractSpace();
-
-    // we found "(" so we got an responseCode    
-    if (parser.startsWith([[40]]))
-    {
-      // remove the opening bracket
-      parser.extract(1);
-        
-      // extract Tokens until a ")" is found
-      this.responseCode = parser.extractToken(41);
-        
-      // remove the closing bracket
-      parser.extract(1);        
-        
-      if (parser.isLineBreak())
-        return;
-             
-      parser.extractSpace();
-    }
-    
-    this.message = parser.extractString();
+  this.message = "";
+  this.responseCode = [];
     
+  // OK
+  if (parser.startsWith([[79,111],[75,107]]))
+  {
+    this.response = 0;
+    parser.extract(2);
+  }
+  // BYE
+  else if (parser.startsWith([[66,98],[89,121],[69,101]]))
+  {
+    this.response = 1;
+    parser.extract(3);
+  }
+  // NO
+  else if (parser.startsWith([[78,110],[79,111]]))
+  {
+    this.response = 2;
+    parser.extract(2);
+  }
+  else
+    throw "NO, OK or BYE expected in "+parser.getData();
+
+  // is there a Message?
+  if (parser.isLineBreak())
+  {
     parser.extractLineBreak();
+    return;
+  }
+
+  // remove the space
+  parser.extractSpace();
+
+  // we found "(" so we got an responseCode, they are extremely ugly...    
+  if (parser.startsWith([[40]]))
+  {
+    // remove the opening bracket...
+    parser.extract(1);
+    // ... but remember it 
+    var nesting = 0;
+  
+    // According to the RFC the first tag must be always an atom, but in...
+    // ... reality this is not true. Cyrus servers send it as a string
+    if (parser.isString())
+      this.responseCode.push(parser.extractString());
+    else
+      this.responseCode.push(parser.extractToken([32,41]));
+          
+    while (parser.isSpace())
+    {
+      parser.extractSpace();
+    
+      // We might stumbe upon opening brackets...
+      if (parser.startsWith([[40]]))
+      {
+        // ... oh we did, so increase our nesting counter.
+        parser.extract(1);
+        nesting++;
+      }
+    
+      // ok, more tokens, more fun...
+      // ... it could be either a string, a number, an atom or even a backet
+      if (parser.isString())
+        this.responseCode.push(parser.extractString());
+      else
+        this.responseCode.push(parser.extractToken([32,41]))
+
+      // is it a closing bracket
+      if (parser.startsWith([[41]]) && nesting)
+      {
+        parser.extract(1);
+        nesting--;
+      }
+    }
+    
+    if (!parser.startsWith([[41]]))
+      throw "Closing Backets expected in "+parser.getData();
+      
+    parser.extract(1);    
+        
+    if (parser.isLineBreak())
+    {
+      parser.extractLineBreak();
+      return
+    }
+             
+    parser.extractSpace();
+  }  
+    
+  this.message = parser.extractString();
+    
+  parser.extractLineBreak();
 }
 
-SieveAbstractResponse.prototype.message = null;
-SieveAbstractResponse.prototype.responseCode = null;
-SieveAbstractResponse.prototype.response = -1;
+SieveSimpleResponse.prototype.message = null;
+SieveSimpleResponse.prototype.responseCode = null;
+SieveSimpleResponse.prototype.response = null;
 
-SieveAbstractResponse.prototype.getMessage
-    = function () { return this.message; }
+SieveSimpleResponse.prototype.getMessage
+    = function () 
+{ 
+  if (this.message == null)
+    throw "Message not Initialized";
+    
+  return this.message; 
+}
 
-SieveAbstractResponse.prototype.hasError
+SieveSimpleResponse.prototype.hasError
     = function ()
 {
+  if (this.response == null)
+    throw "response not Initialized";
+    
   if (this.response == 0)
     return false;
 
   return true;
 }
 
-SieveAbstractResponse.prototype.getResponse
+SieveSimpleResponse.prototype.getResponse
     = function () { return this.response; }
 
-SieveAbstractResponse.prototype.getResponseCode
+SieveSimpleResponse.prototype.getResponseCode
     = function ()
 {
-  // If the Response Code starts with a quote skip we run into a cyrus bug. 
-  // This means we need an offset of 1 first character...  
-  var offset = this.responseCode[0] == '"'?1:0;
-  
-  if (this.responseCode.toUpperCase().indexOf("REFERRAL") == offset)
-    return new SieveResponseCodeReferral(this.responseCode);
+  if (this.responseCode == null)
+    throw "Response Code not Initialized";
     
-  if (this.responseCode.toUpperCase().indexOf("SASL") == offset)
-    return new SieveResponseCodeSasl(this.responseCode);
+  var code = "";  
+  if (this.responseCode.length)
+    code = this.responseCode[0].toUpperCase()
+    
+  switch (code)
+  {
+    case "REFERRAL":
+      return new SieveResponseCodeReferral(this.responseCode);
+      
+    case "SASL":
+      return new SieveResponseCodeSasl(this.responseCode);          
+  }
 
   // TODO Implement these Response codes:
-  //"ACTIVE" / "NONEXISTENT" / "ALREADYEXISTS" / "WARNINGS" 
-    
+  //"ACTIVE" / "NONEXISTENT" / "ALREADYEXISTS" / "WARNINGS"
   return new SieveResponseCode(this.responseCode);
 }
 
-/**
- * This class implements a generic response handler for simple sieve requests.
- * 
- * Simple Requests indicate, wether the command succeded or not. They return
- * only status information, and do not contain any data relevant for the user.
- *  
- * @param {} data
- *  a string containing the response sent by the server
- */
-function SieveSimpleResponse(data)
-{
-  SieveAbstractResponse.call(this,new SieveResponseParser(data));
-}
-
-// Inherrit prototypes from SieveAbstractResponse...
-SieveSimpleResponse.prototype.__proto__ = SieveAbstractResponse.prototype;
-
 /**
  * Parses the capabilites posted by the ManageSieve server upon a client 
  * connection, after successful STARTTLS and AUTHENTICATE or by issuing the 
@@ -156,12 +189,12 @@ SieveSimpleResponse.prototype.__proto__ = SieveAbstractResponse.prototype;
  * 
  * @see {SieveCapabilitiesRequest}
  * 
- * @param {String} data
- *   a string containing the response sent by the server 
+ * @param {SieveResponseParser} parser
+ *   a parser containing the response sent by the server 
  */
-function SieveCapabilitiesResponse(data)
+function SieveCapabilitiesResponse(parser)
 {    
-  this.implementation = "";
+  this.implementation = null;
   this.version = 0;
   
   this.extensions = {};
@@ -175,8 +208,6 @@ function SieveCapabilitiesResponse(data)
   
   this.capabilities = {};
   
-  var parser = new SieveResponseParser(data);
-  
   while (parser.isString() )
   {
     var tag = parser.extractString();
@@ -216,7 +247,7 @@ function SieveCapabilitiesResponse(data)
         
         break;
       case "MAXREDIRECTS":
-        this.maxredirects = parseInt(value);
+        this.maxredirects = parseInt(value,10);
         break;
       case "LANGUAGE":
         this.language = value;
@@ -234,15 +265,17 @@ function SieveCapabilitiesResponse(data)
         this.capabilities.noop = true;
         break;
     }
-    
   } 
 
+  if (this.implementation === null)
+    throw "Implementation expected";
+    
   // invoke inheritted Object constructor...
-  SieveAbstractResponse.call(this,parser);  
+  SieveSimpleResponse.call(this,parser);  
 }
 
-// Inherrit properties from SieveAbstractResponse
-SieveCapabilitiesResponse.prototype.__proto__ = SieveAbstractResponse.prototype;
+// Inherrit properties from SieveSimpleResponse
+SieveCapabilitiesResponse.prototype.__proto__ = SieveSimpleResponse.prototype;
 
 SieveCapabilitiesResponse.prototype.getImplementation
     = function () { return this.implementation; }
@@ -346,16 +379,14 @@ SieveCapabilitiesResponse.prototype.getCapabilities
 SieveCapabilitiesResponse.prototype.getOwner
     = function () { return this.owner; }    
 
-    
+
 //***************************************************************************//
     
-function SieveListScriptResponse(data)
+function SieveListScriptResponse(parser)
 {
   //    sieve-name    = string
   //    string        = quoted / literal
   //    (sieve-name [SP "ACTIVE"] CRLF) response-oknobye
-
-  var parser = new SieveResponseParser(data);
   
   this.scripts = new Array();
   var i = -1;
@@ -377,7 +408,7 @@ function SieveListScriptResponse(data)
         
         parser.extractSpace();
         
-        if (parser.extractToken(13).toUpperCase() != "ACTIVE")
+        if (parser.extractToken([13]).toUpperCase() != "ACTIVE")
             throw "Error \"ACTIVE\" expected";   
 
         this.scripts[i].active = true;        
@@ -386,11 +417,11 @@ function SieveListScriptResponse(data)
     }
 
   // invoke inheritted Object constructor...
-  SieveAbstractResponse.call(this,parser);  
+  SieveSimpleResponse.call(this,parser);  
 }
 
-// Inherrit properties from SieveAbstractResponse
-SieveListScriptResponse.prototype.__proto__ = SieveAbstractResponse.prototype;    
+// Inherrit properties from SieveSimpleResponse
+SieveListScriptResponse.prototype.__proto__ = SieveSimpleResponse.prototype;    
    
 SieveListScriptResponse.prototype.getScripts
     = function () { return this.scripts; }
@@ -399,18 +430,21 @@ SieveListScriptResponse.prototype.getScripts
 //*************************************
 function SieveSaslLoginResponse()
 {
-  this.superior = null;
   this.state = 0;
 }
 
+SieveSaslLoginResponse.prototype.__proto__ = SieveSimpleResponse.prototype;
+
 SieveSaslLoginResponse.prototype.add
-  = function (data) 
+  = function (parser) 
 {
-  var parser = new SieveResponseParser(data);
   
   if ((this.state == 0) && (parser.isString()))
   {
     // String should be 'Username:' or something similar
+    parser.extractString();
+    parser.extractLineBreak();
+
     this.state++;
     return;
   }
@@ -418,6 +452,9 @@ SieveSaslLoginResponse.prototype.add
   if ((this.state == 1) && (parser.isString()))
   {
     // String should be equivalten to 'Password:'
+    parser.extractString();
+    parser.extractLineBreak();    
+
     this.state++;
     return;
   }
@@ -426,81 +463,50 @@ SieveSaslLoginResponse.prototype.add
   {
     // Should be either a NO, BYE or OK
     this.state = 4;
-    this.superior = new SieveAbstractResponse(parser);
+    SieveSimpleResponse.call(this,parser);
     return;
   }
   
   // is it an error message? 
   try
   {
-    this.superior = new SieveAbstractResponse(parser);
+    SieveSimpleResponse.call(this,parser);
     this.state = 4;
     return;
   }
   catch (ex) 
   {
-    throw 'Illegal State:'+this.state+' / '+data+'\n'+ex;    
+    throw 'Illegal State:'+this.state+' / '+parser.getData(0)+'\n'+ex;    
   }
     
-  throw 'Illegal State:'+this.state+' / '+data;
+  throw 'Illegal State:'+this.state+' / '+parser.getData(0);
 }
 
 SieveSaslLoginResponse.prototype.getState
   = function () { return this.state; }
 
-SieveSaslLoginResponse.prototype.getMessage
-  = function ()
-{
-  if (this.state != 4)
-    throw "Illegal State, request not completed";
-      
-  return this.superior.getMessage(); 
-}
-
-SieveSaslLoginResponse.prototype.hasError
-  = function () 
-{
-  if (this.state != 4)
-    throw "Illegal State, request not completed";
-    
-  return this.superior.hasError(); 
-}
-
-SieveSaslLoginResponse.prototype.getResponse
-  = function () 
-{
-  if (this.state != 4)
-    throw "Illegal State, request not completed";
-      
-  return this.superior.getResponse(); 
-}
-
-SieveSaslLoginResponse.prototype.getResponseCode
-  = function () 
-{
-  if (this.state != 4)
-    throw "Illegal State, request not completed";
-    
-  return this.superior.getResponseCode(); 
-}
-
 //*************************************
+/**
+ * @author Thomas Schmid
+ * @author Max Dittrich 
+ */
 function SieveSaslCramMd5Response()
 {
-  this.superior = null;
   this.state = 0;
 }
 
+SieveSaslCramMd5Response.prototype.__proto__ = SieveSimpleResponse.prototype;
+
 SieveSaslCramMd5Response.prototype.add
-  = function (data) 
+  = function (parser) 
 {
   
-  var parser = new SieveResponseParser(data);
-  
   if ((this.state == 0) && (parser.isString()))
   {
     // The challenge is contained within a string
     this.challenge = parser.extractString();
+    parser.extractLineBreak();
+    
     this.state++;
     
     return;
@@ -510,11 +516,13 @@ SieveSaslCramMd5Response.prototype.add
   {
     // Should be either a NO, BYE or OK
     this.state = 4;
-    this.superior = new SieveAbstractResponse(parser);
+    
+    // Invoke the interited constructor to parse the rest of the message
+    SieveSimpleResponse.call(this,parser);
     return;
   }
     
-  throw 'Illegal State:'+this.state+' / '+data;
+  throw 'Illegal State:'+this.state+' / '+parser.getData();
 }
 
 SieveSaslCramMd5Response.prototype.getState
@@ -529,42 +537,6 @@ SieveSaslCramMd5Response.prototype.getChallenge
   return this.challenge; 
 }
 
-SieveSaslCramMd5Response.prototype.getMessage
-  = function ()
-{
-  if (this.state != 4)
-    throw "Illegal State, request not completed";
-      
-  return this.superior.getMessage(); 
-}
-
-SieveSaslCramMd5Response.prototype.hasError
-  = function () 
-{
-  if (this.state != 4)
-    throw "Illegal State, request not completed";
-    
-  return this.superior.hasError(); 
-}
-
-SieveSaslCramMd5Response.prototype.getResponse
-  = function () 
-{
-  if (this.state != 4)
-    throw "Illegal State, request not completed";
-      
-  return this.superior.getResponse(); 
-}
-
-SieveSaslCramMd5Response.prototype.getResponseCode
-  = function () 
-{
-  if (this.state != 4)
-    throw "Illegal State, request not completed";
-    
-  return this.superior.getResponseCode(); 
-}
-
 /*********************************************************
     literal               = "{" number  "+}" CRLF *OCTET
     quoted                = <"> *1024QUOTED-CHAR <">
@@ -572,13 +544,11 @@ SieveSaslCramMd5Response.prototype.getResponseCode
     string                = quoted / literal
 **********************************************************/
 
-function SieveGetScriptResponse(scriptName,data)
+function SieveGetScriptResponse(scriptName,parser)
 {
   /** @private, @type {String} */ this.scriptName = scriptName;
   /** @private, @type {String} */ this.scriptBody = "";
   
-  var parser = new SieveResponseParser(data);
-  
   if (parser.isString())
   {
     this.scriptBody = parser.extractString();
@@ -586,17 +556,18 @@ function SieveGetScriptResponse(scriptName,data)
   }
 	
   // invoke inheritted Object constructor...
-  SieveAbstractResponse.call(this,parser);  
+  SieveSimpleResponse.call(this,parser);  
 }
 
-// Inherrit properties from SieveAbstractResponse
-SieveGetScriptResponse.prototype.__proto__ = SieveAbstractResponse.prototype;    
+// Inherrit properties from SieveSimpleResponse
+SieveGetScriptResponse.prototype.__proto__ = SieveSimpleResponse.prototype;    
 
 /**
- * Conatins the requested sieve script. Script can't be locket, this means
- * several clients can manipulate a script at the same time.
+ * Contains the requested sieve script. 
+ * Keep in mind scripts can't be locked, so several clients may manipulate 
+ * a script at the same time.
  * 
- * @return {String} returns the script's content
+ * @return {String} returns the requested script's content
  */    
 SieveGetScriptResponse.prototype.getScriptBody
     = function () { return this.scriptBody; }
@@ -605,4 +576,224 @@ SieveGetScriptResponse.prototype.getScriptBody
  * @return {String} Containing the script's Name.
  */
 SieveGetScriptResponse.prototype.getScriptName
-    = function () { return this.scriptName; }        
\ No newline at end of file
+    = function () { return this.scriptName; }
+ 
+/**
+ * Parses responses for SCRAM-SHA-1 authentication.
+ * 
+ * SCRAM is a secure client first authentication mechanism. The client
+ * callanges the server and descides if the connection is trustworthy.
+ * 
+ * This requires a way mor logic on the client than with simple authentication
+ * mechanisms. It also requires more communication, in total two roundtrips. 
+ */
+ 
+function SieveSaslScramSha1Response()
+{
+  this.state = 0;
+}
+
+SieveSaslScramSha1Response.prototype.__proto__ = SieveSimpleResponse.prototype;
+
+
+/**
+ * @private
+ * 
+ * Parses the server-first-message it is defined to be:
+ *   [reserved-mext ","] nonce "," salt "," iteration-count ["," extensions]
+ * 
+ * Where 
+ *  reserved-mext   : "m=" 1*(value-char)
+ *  nonce           : "r=" c-nonce
+ *  salt            : "s=" base64(salt)
+ *  iteration-count : "i=" posit-number
+ * 
+ * Extensions are optional and for future use.
+ * Neithe c-nonce nor salt can contain a "," character 
+ * 
+ * @param {} string
+ */
+SieveSaslScramSha1Response.prototype._parseFirstMessage
+  = function (string)
+{
+  this._serverFirstMessage = atob(string);
+  
+  var tokens = this._serverFirstMessage.split(',');
+  
+  // Test for the reserved-mext token. If it is existant, we just skip it
+  if ((tokens[0].length <=2) || tokens[0][0] == "m")
+    tokens.shift();
+  
+  // Extract the nonce
+  if ((tokens[0].length <=2) || (tokens[0][0] != "r"))
+    throw "Nonce missing";
+    
+  this._nonce = tokens[0].substr(2);
+  
+  
+  if ((tokens[1].length <= 2) ||(tokens[1][0] != "s"))
+    throw "Salt missing";
+    
+  this._salt = atob(tokens[1].substr(2));
+  
+  
+  if ((tokens[2].length <= 2) || (tokens[2][0] != "i"))
+    throw "Iteration Count missing";
+    
+  this._iter = parseInt(tokens[2].substr(2),10);
+}
+
+/**
+ * Parses the server-final-message. It is defined to be:
+ *   (server-error / verifier) ["," extensions]
+ * 
+ * Where 
+ *  server-error    : "e=" server-error-value
+ *  verifier        : "v=" base64(ServerSignature)
+ * 
+ * Extensions are optional and for future use.
+ * As suggested by the RFC they will be ignored 
+ * 
+ * @param {} string
+ */
+SieveSaslScramSha1Response.prototype._parseFinalMessage
+  = function (string)
+{ 
+  // server-final-message = (server-error / verifier) ["," extensions]
+  var token = atob(string).split(",");
+  
+  if (token[0].length <= 2)
+    throw "Response expected but got : "+ string;
+    
+  // server-error = "e="
+  if (token[0][0] == "e")
+  {
+    this._serverError = token[0].substr(2);
+    return;
+  }
+
+  // verifier = "v=" base64
+  if (token[0][0] == "v")
+  {
+    this._verifier = atob(token[0].substr(2));
+    return
+  }
+   
+  throw "Invalid Final message";
+}
+
+SieveSaslScramSha1Response.prototype.add
+  = function (parser) 
+{
+  
+  if ((this.state == 0) && (parser.isString()))
+  {
+    this._parseFirstMessage(parser.extractString());
+    parser.extractLineBreak();
+            
+    this.state++;
+    
+    return;
+  }
+
+
+  // There are two valid responses...  
+  // ... either the Server sends us something like that:
+  //
+  //   S: cnNwYXV0aD1lYTQwZjYwMzM1YzQyN2I1NTI3Yjg0ZGJhYmNkZmZmZA==
+  //   C: ""
+  //   S: OK
+  
+  if ((this.state == 1) && (parser.isString()))
+  {
+
+    this._parseFinalMessage(parser.extractString());
+    parser.extractLineBreak();
+      
+    this.state++;
+      
+    return;
+  }
+  
+  // Or the response is wrapped into the ResponseCode in order to save...
+  // ... roundtip time so we endup with the following
+  //
+  // S: OK (SASL "cnNwYXV0aD1lYTQwZjYwMzM1YzQyN2I1NTI3Yjg0ZGJhYmNkZmZmZA==")
+  
+  if (this.state == 1)
+  {
+    SieveSimpleResponse.call(this,parser);
+    
+    this._parseFinalMessage(this.getResponseCode().getSasl())
+    
+    this.state = 4;
+     
+    return;
+  }
+    
+  if (this.state == 2)
+  {
+    SieveSimpleResponse.call(this,parser);
+    this.state = 4;
+    return;
+  }
+     
+  throw 'Illegal State:'+this.state+' / '+parser.getData();
+}
+
+SieveSaslScramSha1Response.prototype.getState
+  = function () { return this.state; }
+
+SieveSaslScramSha1Response.prototype.getSalt
+  = function ()
+{
+  if (this.state < 1)
+    throw "Illegal State, request not completed";
+      
+  return this._salt; 
+}
+
+SieveSaslScramSha1Response.prototype.getIterationCounter
+  = function ()
+{
+  if (this.state < 1)
+    throw "Illegal State, request not completed";
+      
+  return this._iter; 
+}
+
+SieveSaslScramSha1Response.prototype.getNonce
+  = function ()
+{
+  if (this.state < 1)
+    throw "Illegal State, request not completed";
+      
+  return this._nonce; 
+}
+
+SieveSaslScramSha1Response.prototype.getServerFirstMessage
+  = function ()
+{
+  if (this.state < 1)
+    throw "Illegal State, request not completed";
+      
+  return this._serverFirstMessage;     
+}
+
+SieveSaslScramSha1Response.prototype.getServerError
+  = function ()
+{
+  if (this.state < 2)
+    throw "Illegal State, request not completed";
+      
+  return this._serverError;     
+}
+
+SieveSaslScramSha1Response.prototype.getVerifier
+  = function ()
+{
+  if (this.state < 2)
+    throw "Illegal State, request not completed";
+      
+  return this._verifier;     
+}
\ No newline at end of file
