@@ -1,6 +1,12 @@
-/*const Cc = Components.classes;
-const Ci = Components.interfaces;
-const Cr = Components.results;*/
+
+if (typeof(Cc) == 'undefined')
+  { Cc = Components.classes; }
+
+if (typeof(Ci) == 'undefined')
+  { Ci = Components.interfaces; }  
+
+if (typeof(Cr) == 'undefined')
+  { Cr = Components.results; }
 
 Cc["@mozilla.org/moz/jssubscript-loader;1"]
     .getService(Ci.mozIJSSubScriptLoader) 
@@ -37,28 +43,70 @@ Cc["@mozilla.org/moz/jssubscript-loader;1"]
  *   a unique Identifier for this Session. Only neede to make debugging easyer.
  *   
  */
-function SieveSession(account,sid)
+function SieveSession(accountId,sid)
 {
   this.idx = 0;
 
-  this.account = account;
+  // Load Account by ID
+  this.account = (new SieveAccounts()).getAccount(accountId);
   
   this.debug = {};
   this.debug.level = this.account.getSettings().getDebugFlags();
-  this.debug.logger = Cc["@mozilla.org/consoleservice;1"]
-                          .getService(Ci.nsIConsoleService);
+  
+  // Use an empty logger stub, it makes the logger easily exchangable...
+  // ... this.logger.logString is garanteed to exist within this file.
+  this.debug.logger = {}
+  //this.debug.logger.logStringMessage = function(msg) { };  
+
+  /*this.debug.logger = Cc["@mozilla.org/consoleservice;1"]
+    .getService(Ci.nsIConsoleService);*/
+  
+  /*this.debug.logger = {}
+  this.debug.logger.logStringMessage = function(msg) {
+    Cc["@mozilla.org/embedcomp/prompt-service;1"]
+      .getService(Components.interfaces.nsIPromptService)
+      .alert(null, "Alert", msg);
+  }*/
+  
+  this.debug.logger = {};
+  this.debug.logger._sid = sid;
+  this.debug.logger._pad = function(n,m) {
+    var str = n;
+  
+    for (var i = 0; i < m; i++)
+      if (n < Math.pow(10,i))
+        str = '0'+str;
+
+     return str;    
+  };
+  
+  this.debug.logger.getTimestamp = function () {
+    var date = new  Date();
+    return this._pad(date.getHours(),2)
+      + ":"+this._pad(date.getMinutes(),2)
+      + ":"+this._pad(date.getSeconds(),2)
+      + "."+this._pad(date.getMilliseconds(),3);
+  };
+  
+  this.debug.logger.logStringMessage = function(msg) {
+    Cc["@mozilla.org/consoleservice;1"]
+      .getService(Ci.nsIConsoleService)
+      .logStringMessage("["+this.getTimestamp()+" "+this._sid+"] "+msg);
+  };
   
   this.sid = sid;
+  
+  // 0 = Offline 1 = Connecting 2 = connected 3= disconnecting 
+  this.state = 0;
 }
 
 SieveSession.prototype = 
-{  
+{
   onIdle: function ()
-  { 
-    // redirect to window managing the session...
-    /*if ((this.listener) && (this.listener.onIdle))
-     this.listener.onIdle(this.sieve);*/
- 
+  {
+    if (this.debug.level & (1 << 2))
+      this.debug.logger.logStringMessage("Sending keep alive packet...");
+      
     // as we send a keep alive request, we don't care
     // about the response...
     var request = null;
@@ -67,8 +115,8 @@ SieveSession.prototype =
       request = new SieveNoopRequest();
     else
       request = new SieveCapabilitiesRequest();
-  
-    this.sieve.addRequest(request);      
+    
+    this.sieve.addRequest(request);
   },
 
   onInitResponse: function(response)
@@ -77,25 +125,29 @@ SieveSession.prototype =
     // ... is capable of handling TLS, otherwise simply skip it and ...
     // ... use an insecure connection
     
-    this.sieve.setCompatibility(response.getCapabilities());
-    
-    if (this.account.getHost().isTLS() && response.getTLS())
+    if (!this.account.getHost().isTLSEnabled())
     {
-      var request = new SieveStartTLSRequest();
-      request.addStartTLSListener(this);
-      request.addErrorListener(this);
-      
-      this.sieve.addRequest(request);
-      
+      this.onAuthenticate(response);
       return;
     }
     
-    this.onAuthenticate(response);
+    if (!response.getTLS() && !this.account.getHost().isTLSForced())
+    {
+      this.onAuthenticate(response);
+      return;
+    }
+    
+    var request = new SieveStartTLSRequest();
+    request.addStartTLSListener(this);
+    request.addErrorListener(this);
+    
+    this.sieve.addRequest(request);
   },
   
   onAuthenticate: function(response)
-  {
-    this.listener.onChannelStatus(3,"progress.authenticating");
+  { 
+    this._invokeListeners("onChannelStatus",3,"progress.authenticating");
+      
     var account =  this.account;
     
     // Without a username, we can skip the authentication 
@@ -108,8 +160,7 @@ SieveSession.prototype =
 
     // Notify the listener to display capabilities. We simply pass the response... 
     // ... to the listener. So the listener can pick whatever he needs.
-    if (this.listener && this.listener.onChannelStatus)
-      this.listener.onChannelStatus(7,response);
+    this._invokeListeners("onChannelStatus",7,response);
 
     // We have to figure out which ist the best SASL Mechanism for the login ...
     // ... therefore we check first whether a mechanism is forced by the user ...
@@ -122,36 +173,39 @@ SieveSession.prototype =
     else
       mechanism = response.getSasl();
 
-    // ... translate the SASL Mechanism String into an SieveSaslLogin Object ...      
+    // ... translate the SASL Mechanism into an SieveSaslLogin Object ...      
     var request = null;
     while((mechanism.length > 0) && (request == null))
     {
       // remove and test the first element...
-      switch (mechanism.shift().toLowerCase())
+      switch (mechanism.shift().toUpperCase())
       {
-        case "plain":
+        case "PLAIN":
           request = new SieveSaslPlainRequest();
-          request.addSaslPlainListener(this);      
           break;
-        case "crammd5":
+          
+        case "CRAM-MD5":
           request = new SieveSaslCramMd5Request();
-          request.addSaslCramMd5Listener(this);
-          break;          
-        case "login":
+          break;
+          
+        case "SCRAM-SHA-1":
+          request = new SieveSaslScramSha1Request();
+          break;
+          
+        case "LOGIN":
           // we use SASL LOGIN only as last resort...
           // ... as suggested in the RFC.        
           if (mechanism.length > 0)
           {
-            mechanism.push("login");
+            mechanism.push("LOGIN");
             break;
           }
           request = new SieveSaslLoginRequest();      
-          request.addSaslLoginListener(this);
           break;
       }      
     }
 
-    if (request == null)
+    if (!request)
     {
       this.disconnect(false,2,"error.sasl");
       return;
@@ -168,6 +222,7 @@ SieveSession.prototype =
       return;
     }
       
+    request.addSaslListener(this);
     request.setPassword(password);
     
     // check if the authentication method supports proxy authorization...
@@ -181,123 +236,106 @@ SieveSession.prototype =
         return;
       }
       
-      request.setAuthorization(authorization);
+      if (authorization != "")
+        request.setAuthorization(authorization);
     }
      
-    this.sieve.addRequest(request);       
-    
+    this.sieve.addRequest(request);    
   },
 
   onStartTLSResponse : function(response)
-  {
-    // As we are an object we can't use this...
-    // it would refer to lEvent instead of SieveSession
+  { 
+    this.sieve.startTLS();
+
     var that = this;
-    // workaround for timsieved bug...
+    
     var lEvent = 
     {
-      onInitResponse: function(response)
-      {
-        that.listener.onChannelStatus(3,"progress.tls.rfc");
-        
-        that.sieve.setTimeoutInterval();
-        that.onAuthenticate(response);
-      },
-      
-      onError: function(response)
-      {
-        that.sieve.setTimeoutInterval();
-        that.onError(response);
-      },
-      
-      onTimeout: function()
-      {
-        that.listener.onChannelStatus(3,"progress.tls.cyrus");
-        
-        that.sieve.setTimeoutInterval();
-        var request = new SieveCapabilitiesRequest();
-        request.addCapabilitiesListener(that);
-        request.addErrorListener(that);  
-    
-        that.sieve.addRequest(request);
-      }     
+      onCapabilitiesResponse: function(response)
+      {        
+        that.onAuthenticate(response); 
+      }
     }
-        
-    // after calling startTLS the server will propagate his capabilities...
-    // ... like at the login, therefore we reuse the SieveInitRequest
     
-    // some revision of timsieved fail to resissue the capabilities...
-    // ... which causes the extension to be jammed. Therefore we have to ...
-    // ... do a rather nasty workaround. The jammed extension causes a timeout,
-    // ... we catch this timeout and continue as if nothing happend...
+    // explicitely request capabilites
+    var request = new SieveCapabilitiesRequest();
+    request.addCapabilitiesListener(lEvent);
     
-    var compatibility = this.account.getSettings().getCompatibility(); 
+    this.sieve.addRequest(request);
     
-    switch (compatibility.getHandshakeMode())
-    {
-      case 0:
-        this.listener.onChannelStatus(3,"progress.tls.auto");      
-        var request = new SieveInitRequest();
-        request.addInitListener(lEvent);
-        request.addErrorListener(lEvent);
-        
-        this.sieve.setTimeoutInterval(compatibility.getHandshakeTimeout());
-        this.sieve.addRequest(request);
-            
-        this.sieve.startTLS();   
-        break;
-        
-      case 1:
-        this.listener.onChannelStatus(3,"progress.tls.rfc");
-             
-        var request = new SieveInitRequest();
-        request.addInitListener(lEvent);
-        request.addErrorListener(this);  
-        this.sieve.addRequest(request);
-    
-        // activate TLS
-        this.sieve.startTLS();
-        
-        break;
-      case 2:
-        this.listener.onChannelStatus(3,"progress.tls.cyrus");
-        //sivSetStatus(3,"progress.tls.cyrus");
-      
-        this.sieve.startTLS();
-      
-        var request = new SieveCapabilitiesRequest();
-        request.addCapabilitiesListener(this);
-        request.addErrorListener(this);  
-    
-        this.sieve.addRequest(request);
-        break;
-    }
+    // With a bugfree server we endup with two capability request, one 
+    // implicit after startTLS and one explicite from capbilites. So we have
+    // to consume one of them silently...
+    this.sieve.addRequest(new SieveInitRequest(),true);
   },
   
-  onCapabilitiesResponse: function(response)
-  {
-    this.onAuthenticate(response);
-  },
-  
-  onSaslLoginResponse: function(response)
+  onSaslResponse: function(response)
   {
     this.onLoginResponse(response);
   },
   
-  onSaslPlainResponse: function(response)
-  {
-    this.onLoginResponse(response);
-  },
-
-  onSaslCramMd5Response: function(response)
-  {
-    this.onLoginResponse(response);
-  },
-   
   onLoginResponse: function(response)
   {
-    if (this.listener)
-      this.listener.onChannelCreated(this.sieve);    
+    // We are connected...
+    this.state = 2;
+    this._invokeListeners("onChannelCreated",this.sieve);
+  },
+  
+  addListener: function(listener)
+  {
+    if (!this.listeners)
+      this.listeners = [];
+      
+    this.listeners.push(listener);
+    
+  },
+  
+  removeListener: function(listener)
+  {
+    if (!this.listeners)
+      return;
+
+    for (var i=0; i< this.listeners.length; i++)
+      if (this.listeners[i] == listener)
+        this.listeners.splice(i,1);
+ 
+  },
+  
+  _hasListeners: function(callback)
+  {
+    if (!this.listeners)
+      return false;
+      
+    for (var i=0; i< this.listeners.length; i++)
+     if (this.listeners[i][callback])
+       return true;      
+  },
+  
+  _invokeListeners: function(callback,arg1, arg2)
+  {
+    if (!this.listeners)
+      return false;
+
+    // the a callback function might manipulate our listeners...
+    // ... so we need to cache them before calling...      
+    var iterator = [];
+    for (var i=0; i< this.listeners.length; i++)
+      if (this.listeners[i][callback])
+        iterator.push(this.listeners[i][callback]);
+       
+    if (!iterator.length)
+    {
+      if (this.debug.level & (1 << 4))
+        this.debug.logger.logStringMessage("No Listener for "+callback+"\n"+this.listeners.toSource());
+      return;
+    }
+    
+    if (this.debug.level & (1 << 4))
+      this.debug.logger.logStringMessage("Invoking Listeners for "+callback+"\n");
+      
+    while (iterator.length)
+      iterator.pop()(arg1,arg2);
+      
   },
 
   /** @private */  
@@ -315,11 +353,14 @@ SieveSession.prototype =
     // The server is going to disconnected our session nicely...
     var code = response.getResponseCode();
     
+    // ... as the server must terminate the connection after sending a ...
+    // ... bye response, we should also disconnect nicely which means in this...
+    // ... case without a logout request.
+    this.disconnect(true);
+    
     // ... we most likely received a referal 
     if (code.equalsCode("REFERRAL"))
     {
-      this.disconnect(true);
-      
       if (this.debug.level & (1 << 4))
         this.debug.logger.logStringMessage("Referred to Server: "+code.getHostname());
       
@@ -340,10 +381,31 @@ SieveSession.prototype =
   },
 
   /** @private */
-  onTimeout: function()
+  onTimeout: function(message)
   {
-    if (this.listener && this.listener.onTimeout)
-      this.listener.onTimeout();    
+    var ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);  
+    
+    if (ioService.offline)
+    {
+      this._invokeListeners("onOffline");
+      return;
+    }
+    
+    this._invokeListeners("onTimeout",message);
+  },
+  
+  /**
+   * This listener is called when the conenction is lost or terminated by the server
+   * and the session is no more usable. It ensures that everything is disconnected
+   * correctly.
+   **/
+  onDisconnect: function()
+  {    
+    if (this.debug.level & (1 << 4))
+      this.debug.logger.logStringMessage("On Server Disconnect: "+this.sid+"/ ["+this.channels+"]");
+        
+    this._invokeListeners("onDisconnect");  
+    this.disconnect(true);
   },
   
   /**
@@ -362,13 +424,10 @@ SieveSession.prototype =
    *   overrides the default port supplied by the account.
    */    
   connect : function(hostname,port)
-  {                   
-    if (this.sieve && this.sieve.isAlive())
-    {        
-      this.onLoginResponse(null);     
-      return;
-    }
-        
+  { 
+    // set state to connecting...
+    this.state = 1;
+    
     this.sieve = Cc["@sieve.mozdev.org/transport;1"]
                    .createInstance().wrappedJSObject;
     
@@ -379,9 +438,8 @@ SieveSession.prototype =
                    
     this.sieve.addListener(this);
   
-    // TODO Load Timeout interval from account settings...
-    
-    if (this.account.getSettings().isKeepAlive(this.account.getSettings()))
+    // TODO Load Timeout interval from account settings...    
+    if (this.account.getSettings().isKeepAlive())
       this.sieve.setKeepAliveInterval(this.account.getSettings().getKeepAliveInterval());    
     
     // Step 2: Initialize Message Queue...
@@ -399,24 +457,30 @@ SieveSession.prototype =
       
     this.sieve.connect(
         hostname,port,
-        this.account.getHost().isTLS(),
+        this.account.getHost().isTLSEnabled(),
         this,
         this.account.getProxy().getProxyInfo());    
   },
   
   disconnect : function(force, id, message)
   {
+    // update state we are now disconnecting...
+    this.state = 3;
+    
     // at first we update the status, so that the user ...
     // ... knows what happened.
-    if (id && this.listener && this.listener.onChannelStatus)
-      this.listener.onChannelStatus(id,message);
-    
+    if (id)
+      this._invokeListeners("onChannelStatus",id,message);
+
     // Skip if the connection already closed...
     if (!this.sieve)
-      return;
-      
-    // ... we always try to exit with an Logout request...
-    if ( !force && this.sieve.isAlive())
+    {
+      this.state = 0;
+      return
+    }
+    
+    // ... we always try to exit with an Logout request...      
+    if (!force && this.sieve.isAlive())
     {
       var request = new SieveLogoutRequest();
       request.addLogoutListener(this);
@@ -429,9 +493,34 @@ SieveSession.prototype =
     // ... but this obviously is not always usefull
     this.sieve.disconnect();      
     this.sieve = null;
+    
+    this.channels = null;
+    
+    // update state: we are disconnected
+    this.state = 0;
       
     return;
   },
+
+  isConnecting : function()
+  {
+    return (this.state == 1);
+  },
+
+  isConnected : function()
+  {
+    return (this.state == 2);
+  },
+
+  isDisconnecting : function()
+  {
+    return (this.state == 3);
+  },
+  
+  isDisconnected : function()
+  {
+    return (this.state == 0);
+  },
     
   /**
    * Requests a channel for to this session. After use you have to revoke it via
@@ -456,7 +545,7 @@ SieveSession.prototype =
      
     return cid;
   },
-  
+    
   /**
    * Closes and Invalidates a channel. In case all channels of a session are
    * closed, the connection to the remote server will be terminated. That's why
@@ -558,12 +647,11 @@ SieveSession.prototype =
     if (this.debug.logger != null)
       this.debug.logger.logStringMessage("Sieve BadCertHandler: notifyCertProblem");
   
-    // no listener registert, show the default UI 
-    if ( !(this.listener) || !(this.listener.onBadCert))
+    // no listener registert, show the default UI
+    if (!this._hasListeners("onBadCert"))
       return false;
       
-    // otherwise call the listener and supress the default UI
-    this.listener.onBadCert(targetSite,sslStatus);
+    this._invokeListeners("onBadCert",targetSite,sslStatus);
     return true;
   },
   
@@ -581,12 +669,12 @@ SieveSession.prototype =
       this.debug.logger.logStringMessage("Sieve BadCertHandler: notifySSLError");
     
     // no listener registert, show the default UI 
-    if ( !(this.listener) || !(this.listener.onBadCert))
+    if (!this._hasListeners("onBadCert"))
       return false;
 
     // otherwise call the listener and supress the default UI
-    this.listener.onBadCert(targetSite,error);
-    return true;
+    this._invokeListeners("onBadCert",targetSite,error);
+    return true;      
   },
   
   // Ci.nsIBadCertListener (Thunderbird 2 / Gecko 1.8.x compatibility)
@@ -645,5 +733,5 @@ SieveSession.prototype =
     if (this.debug.logger != null)
       this.debug.logger.logStringMessage("Sieve BadCertHandler: notifyCrlNextupdate");
   }  
-
+  
 }
\ No newline at end of file
